#include "Twiddlerino.h"
#include "PID_v1.h"

//Define Variables we'll be connecting to
double target=0;
double pos = 0;
double PWMOut = 0;
long last_time = 0;
long updateInterval = 10; //send position every updateInterval ms

double p = 1.2;
double i = 0;
double d = 0.02;
PID myPID(&pos, &PWMOut, &target, p, i, d, REVERSE);

//handle impulses
boolean impulseOn = false;
long impulseStart = 0;
long impulseLength = 10; //ms


void setup()
{
  Serial.begin(115200);
  TwiddlerinoInit();
  target = 0;
  myPID.SetOutputLimits(-255, 255); //for PWM
  myPID.SetMode(AUTOMATIC);
  myPID.SetSampleTime(1); //every 1 ms
}

void loop()
{
  pos = ReadEncoder();
  long t = millis();
  long dt = t - last_time;
  if (dt >= updateInterval)
  {
    Serial.println((int)pos);
    last_time = t;
  }
  
  //Impulse when the ball is hit
  byte b = 0;
  while(Serial.available() > 0)
  {
    b = Serial.read();
  }
  if (b == 'i')
  {
    impulseOn = true;
    impulseStart = millis();
  }
  
  myPID.Compute();
  
  //render a BASIC impulse by setting PWMOut to max
  // in opposite direction of its current value
  if (impulseOn)
  {
    if ((t - impulseStart) >= impulseLength)
    {
      impulseOn = false;
    } else {
      if (PWMOut >= 0)
      {      
        PWMOut = -255;
      } else {
         PWMOut = 255;
      }
    }
  }
  
  //Write to Twiddlerino
  SetPWMOut(PWMOut);
}
