import processing.serial.*;
import ddf.minim.*;

// Updated for Processing 3.0

//sounds
Minim minim;
AudioPlayer toc;

//constants
int PADDLE_WIDTH = 15;
int PADDLE_HEIGHT = 100;
int BALL_RADIUS = 10;
int BALL_DIAMETER = BALL_RADIUS*2;
int WIDTH = 800;
int HEIGHT = 600;
int DEFAULT_BALL_V = 350;//px per s
int COMPUTER_V = 300; //px per s, make this SLOWER THAN THE BALL
int player_x = 50;
int computer_x = WIDTH-50-PADDLE_WIDTH;
int SCORE_FROM_MIDDLE = 150;

//variables
int ball_x, ball_y, player_y, computer_y;
int ball_vx = DEFAULT_BALL_V;
int ball_vy = DEFAULT_BALL_V;
int player_score = 0;
int computer_score =0;
int twiddlerPosition = 0;
boolean paused = true;
int last_time = 0;


String serialPortName = "/dev/tty.usbserial-A9007Nvd";
//String serialPortName = "COM3";

int SERIAL_WRITE_LENGTH = 32;

Serial myPort;
void settings() 
{
  size(WIDTH,HEIGHT);  
}

void setup()
{
  frameRate(30);
  myPort = new Serial(this, serialPortName, 115200);
  
  //for sounds
  minim = new Minim(this);
  toc = minim.loadFile("toc.wav");
  
  randomSeed(millis());
  
  InitializeGame();
}


void draw() {
  background(255);
  fill(0, 0, 0);
  stroke(0,0,255);
  
  UpdateGame();
  DrawGame();
}

/*
  Detailed functions
*/

void InitializeGame()
{
  paused = true;
  ball_x = (int)random(WIDTH/4, WIDTH*3/4);
  ball_y = (int)random(HEIGHT/4, HEIGHT*3/4);;
  ball_vx = DEFAULT_BALL_V;
  ball_vy = DEFAULT_BALL_V;
  computer_y = ball_y - PADDLE_HEIGHT/2;
  last_time = millis();
}

void UpdateGame()
{
   int t = millis();
   int dt = t - last_time;

   /*
    * UPDATE PLAYER
    */
   ReadTwiddler();
   player_y = min(HEIGHT-PADDLE_HEIGHT, max(0, HEIGHT/2-PADDLE_HEIGHT/2+twiddlerPosition/3));
    
  if (!paused)
   {
     /*
      * UPDATE BALL
      * NOTE: This collision detection is EXTREMELY RUDIMENTARY
      * and relies on small time steps.
      */
     //ball bouncing off of the top/bottom
     if (ball_y < 0 || ball_y > HEIGHT)
     {
       ball_vy *= -1;
     }
     
     //ball hitting paddles
     if (ball_x - BALL_RADIUS  < player_x + PADDLE_WIDTH && ball_x + BALL_RADIUS > player_x
         && ball_y - BALL_RADIUS < player_y + PADDLE_HEIGHT && ball_y +BALL_RADIUS > player_y)
       {
              ball_vx *= -1;
              toc.play(0); //0 means start at position 0
              myPort.write('i'); //send impulse to Twiddlerino
       }
       else if (ball_x - BALL_RADIUS  < computer_x + PADDLE_WIDTH && ball_x + BALL_RADIUS > computer_x
         && ball_y - BALL_RADIUS < computer_y + PADDLE_HEIGHT && ball_y +BALL_RADIUS > computer_y)
       {
              ball_vx *= -1;
              toc.play(0); //0 means start at position 0
              myPort.write('i'); //send impulse to Twiddlerino
       }
       
      /*
       * CHECK FOR WIN CONDITION
       */
     if (ball_x + BALL_RADIUS < 0)
     {
       //computer point
       computer_score++;
       InitializeGame();
     } else if (ball_x - BALL_RADIUS > WIDTH)
     {
       //player point
       player_score++;
       InitializeGame();
     }
     
     ball_x += (int)(ball_vx * dt/1000.0); //1000.0 to convert ms -> s
     ball_y += (int)(ball_vy * dt/1000.0); //1000.0 to convert ms -> s
     
     
      /*
      * UPDATE COMPUTER
      */
      int computer_v = COMPUTER_V;
      if (ball_y < computer_y + PADDLE_HEIGHT/2)
      {
        computer_v *= -1;
      }
      computer_y = min(HEIGHT-PADDLE_HEIGHT, max(0, computer_y+(int)(computer_v * dt/1000.0)));
   }
   last_time = t;
}

void mousePressed()
{
  paused = !paused;
}

void ReadTwiddler()
{
    String inputString = "";
    
    byte input[] = new byte[256];
    while(myPort.available() > 0)
    {
      input = myPort.readBytes();
     }
     if (input != null)
     {
       inputString = new String(input);
       String[] inputStrings = inputString.split("\r\n");
       if (inputStrings.length >= 2)
       {
         try {
         twiddlerPosition = Integer.parseInt(inputStrings[inputStrings.length-2]);
         } finally {}
       }
     }
}

void DrawGame()
{
  background(0); //black bg
  
  fill(255); //white fill
  stroke(255);  //white stroke
  
  ellipse(ball_x, ball_y, BALL_DIAMETER, BALL_DIAMETER);
  
  textSize(36);
  text(player_score, WIDTH/2-SCORE_FROM_MIDDLE, HEIGHT/2);
  text(computer_score, WIDTH/2+SCORE_FROM_MIDDLE, HEIGHT/2);
  
  if (paused)
  {
    text("CLICK TO PLAY", WIDTH/2-150, HEIGHT*7/8);
  }
 
  rect(player_x, player_y, PADDLE_WIDTH, PADDLE_HEIGHT);
  rect(computer_x, computer_y, PADDLE_WIDTH, PADDLE_HEIGHT); 
  
}